#!/usr/bin/env python3
"""CLI command handlers for socialia."""

import json
import sys

from .. import __version__
from ..twitter import Twitter
from ..linkedin import LinkedIn
from ..reddit import Reddit
from ..analytics import GoogleAnalytics
from ..youtube import YouTube


def get_client(platform: str):
    """Get platform client instance."""
    if platform == "twitter":
        return Twitter()
    elif platform == "linkedin":
        return LinkedIn()
    elif platform == "reddit":
        return Reddit()
    elif platform == "youtube":
        return YouTube()
    else:
        raise ValueError(f"Unsupported platform: {platform}")


def cmd_post(args, output_json: bool = False) -> int:
    """Handle post command."""
    if args.file:
        if not args.file.exists():
            print(f"Error: File not found: {args.file}", file=sys.stderr)
            return 1
        text = args.file.read_text().strip()
    elif args.text:
        text = args.text
    else:
        print("Error: Provide text or --file", file=sys.stderr)
        return 1

    if args.dry_run:
        print("=== DRY RUN ===")
        print(f"Platform: {args.platform}")
        if args.platform == "reddit":
            print(f"Subreddit: r/{getattr(args, 'subreddit', 'test')}")
            title = getattr(args, "title", None) or text.split("\n")[0][:100]
            print(f"Title: {title}")
        elif args.platform == "youtube":
            video = getattr(args, "video", None)
            print(f"Video: {video or 'None (community post)'}")
            print(
                f"Title: {getattr(args, 'title', None) or text.split(chr(10))[0][:100]}"
            )
            print(f"Privacy: {getattr(args, 'privacy', 'public')}")
        print(
            f"Text ({len(text)} chars): {text[:100]}{'...' if len(text) > 100 else ''}"
        )
        return 0

    client = get_client(args.platform)
    if args.platform == "twitter":
        result = client.post(
            text,
            reply_to=getattr(args, "reply_to", None),
            quote_tweet_id=getattr(args, "quote", None),
        )
    elif args.platform == "reddit":
        result = client.post(
            text,
            subreddit=getattr(args, "subreddit", "test"),
            title=getattr(args, "title", None),
        )
    elif args.platform == "youtube":
        video_path = getattr(args, "video", None)
        tags = getattr(args, "tags", None)
        result = client.post(
            text,
            video_path=str(video_path) if video_path else None,
            title=getattr(args, "title", None),
            tags=tags.split(",") if tags else None,
            privacy_status=getattr(args, "privacy", "public"),
            thumbnail_path=str(getattr(args, "thumbnail", None))
            if getattr(args, "thumbnail", None)
            else None,
        )
    else:
        result = client.post(text)

    if output_json:
        print(json.dumps(result, indent=2))
    elif result["success"]:
        print("Posted successfully!")
        print(f"ID: {result['id']}")
        print(f"URL: {result['url']}")
    else:
        print(f"Error: {result['error']}", file=sys.stderr)
        return 1

    return 0


def cmd_delete(args, output_json: bool = False) -> int:
    """Handle delete command."""
    client = get_client(args.platform)
    result = client.delete(args.post_id)

    if output_json:
        print(json.dumps(result, indent=2))
    elif result["success"]:
        print(f"Deleted: {args.post_id}")
    else:
        print(f"Error: {result['error']}", file=sys.stderr)
        return 1

    return 0


def cmd_thread(args, output_json: bool = False) -> int:
    """Handle thread command."""
    if not args.file.exists():
        print(f"Error: File not found: {args.file}", file=sys.stderr)
        return 1

    content = args.file.read_text()
    tweets = [t.strip() for t in content.split("---") if t.strip()]

    if not tweets:
        print("Error: No content found in file", file=sys.stderr)
        return 1

    if args.dry_run:
        print("=== DRY RUN (Thread) ===")
        print(f"Platform: {args.platform}")
        print(f"Posts: {len(tweets)}")
        for i, t in enumerate(tweets, 1):
            print(f"\n--- Post {i} ({len(t)} chars) ---")
            print(t[:200] + ("..." if len(t) > 200 else ""))
        return 0

    client = get_client(args.platform)
    result = client.post_thread(tweets)

    if output_json:
        print(json.dumps(result, indent=2))
    elif result["success"]:
        print(f"Thread posted! ({len(result['ids'])} posts)")
        for url in result["urls"]:
            print(f"  {url}")
    else:
        print(f"Error: {result['error']}", file=sys.stderr)
        return 1

    return 0


def cmd_analytics(args, output_json: bool = False) -> int:
    """Handle analytics command."""
    ga = GoogleAnalytics()

    if args.analytics_command == "track":
        params = {}
        if args.param:
            for key, value in args.param:
                params[key] = value
        result = ga.track_event(args.event_name, params)

    elif args.analytics_command == "realtime":
        result = ga.get_realtime_users()

    elif args.analytics_command == "pageviews":
        result = ga.get_page_views(
            start_date=args.start,
            end_date=args.end,
            page_path=getattr(args, "path", None),
        )

    elif args.analytics_command == "sources":
        result = ga.get_traffic_sources(
            start_date=args.start,
            end_date=args.end,
        )

    else:
        print(
            "Error: Specify analytics subcommand (track, realtime, pageviews, sources)",
            file=sys.stderr,
        )
        return 1

    if output_json:
        print(json.dumps(result, indent=2))
    elif result["success"]:
        if args.analytics_command == "track":
            print(f"Event tracked: {args.event_name}")
        elif args.analytics_command == "realtime":
            print(f"Active users: {result.get('active_users', 0)}")
        elif args.analytics_command == "pageviews":
            print(f"Page views ({result['date_range']}):")
            for page in result.get("pages", [])[:10]:
                print(
                    f"  {page['path']}: {page['page_views']} views, {page['users']} users"
                )
        elif args.analytics_command == "sources":
            print(f"Traffic sources ({result['date_range']}):")
            for src in result.get("sources", [])[:10]:
                print(f"  {src['source']}/{src['medium']}: {src['sessions']} sessions")
    else:
        print(f"Error: {result['error']}", file=sys.stderr)
        return 1

    return 0


def cmd_status(output_json: bool = False) -> int:
    """Show configuration and environment status."""
    from .._branding import get_env, get_env_var_name

    env_vars = {
        "twitter": {
            get_env_var_name("X_CONSUMER_KEY"): get_env("X_CONSUMER_KEY"),
            get_env_var_name("X_CONSUMER_KEY_SECRET"): get_env("X_CONSUMER_KEY_SECRET"),
            get_env_var_name("X_ACCESSTOKEN"): get_env("X_ACCESSTOKEN"),
            get_env_var_name("X_ACCESSTOKEN_SECRET"): get_env("X_ACCESSTOKEN_SECRET"),
        },
        "linkedin": {
            get_env_var_name("LINKEDIN_ACCESS_TOKEN"): get_env("LINKEDIN_ACCESS_TOKEN"),
        },
        "reddit": {
            get_env_var_name("REDDIT_CLIENT_ID"): get_env("REDDIT_CLIENT_ID"),
            get_env_var_name("REDDIT_CLIENT_SECRET"): get_env("REDDIT_CLIENT_SECRET"),
            get_env_var_name("REDDIT_USERNAME"): get_env("REDDIT_USERNAME"),
            get_env_var_name("REDDIT_PASSWORD"): get_env("REDDIT_PASSWORD"),
        },
        "youtube": {
            get_env_var_name("YOUTUBE_CLIENT_SECRETS_FILE"): get_env(
                "YOUTUBE_CLIENT_SECRETS_FILE"
            ),
        },
        "analytics": {
            get_env_var_name("GOOGLE_ANALYTICS_MEASUREMENT_ID"): (
                get_env("GOOGLE_ANALYTICS_MEASUREMENT_ID")
                or get_env("GA_MEASUREMENT_ID")
            ),
            get_env_var_name("GOOGLE_ANALYTICS_API_SECRET"): (
                get_env("GOOGLE_ANALYTICS_API_SECRET") or get_env("GA_API_SECRET")
            ),
            get_env_var_name("GOOGLE_ANALYTICS_PROPERTY_ID"): (
                get_env("GOOGLE_ANALYTICS_PROPERTY_ID") or get_env("GA_PROPERTY_ID")
            ),
        },
    }

    status = {}
    for platform, vars_dict in env_vars.items():
        configured = all(v is not None for v in vars_dict.values())
        partial = any(v is not None for v in vars_dict.values())
        status[platform] = {
            "configured": configured,
            "partial": partial and not configured,
            "vars": {k: "set" if v else "missing" for k, v in vars_dict.items()},
        }

    if output_json:
        print(json.dumps({"version": __version__, "platforms": status}, indent=2))
    else:
        print(f"Socialia v{__version__}")
        print("=" * 40)
        for platform, info in status.items():
            if info["configured"]:
                mark = "[OK]"
            elif info["partial"]:
                mark = "[PARTIAL]"
            else:
                mark = "[NOT SET]"
            print(f"\n{platform.upper():12} {mark}")
            for var, state in info["vars"].items():
                indicator = "+" if state == "set" else "-"
                print(f"  {indicator} {var}")

    return 0


def cmd_mcp(args) -> int:
    """Handle MCP command."""
    if args.mcp_command == "run":
        import asyncio
        from ..mcp_server import main as mcp_main, HAS_MCP

        if not HAS_MCP:
            print(
                "Error: MCP package not installed. Run: pip install mcp",
                file=sys.stderr,
            )
            return 1
        asyncio.run(mcp_main())
        return 0

    elif args.mcp_command == "info":
        print("Socialia MCP Server")
        print("=" * 40)
        print()
        print("Available tools:")
        for tool_name in [
            "social_post",
            "social_delete",
            "analytics_track",
            "analytics_realtime",
            "analytics_pageviews",
            "analytics_sources",
        ]:
            print(f"  - {tool_name}")
        print()
        print("Usage:")
        print("  socialia mcp run")
        print("  python -m socialia.mcp_server")
        return 0

    else:
        print("Error: Specify mcp subcommand (run, info)", file=sys.stderr)
        return 1


def _get_setup_guide(platform: str) -> str:
    """Get setup guide with branded env var names."""
    from .._branding import get_env_var_name

    guides = {
        "twitter": f"""
TWITTER/X SETUP
===============

1. Go to https://developer.x.com
2. Create app with Read+Write permissions
3. Generate API keys and access tokens

Environment Variables:
  export {get_env_var_name("X_CONSUMER_KEY")}="..."
  export {get_env_var_name("X_CONSUMER_KEY_SECRET")}="..."
  export {get_env_var_name("X_ACCESSTOKEN")}="..."
  export {get_env_var_name("X_ACCESSTOKEN_SECRET")}="..."

Test:
  socialia post twitter "Test" --dry-run
""",
        "linkedin": f"""
LINKEDIN SETUP
==============

1. Go to https://www.linkedin.com/developers/
2. Create app (or use existing one)
3. Go to Products tab, add TWO products:
   - "Share on LinkedIn" (for posting)
   - "Sign In with LinkedIn using OpenID Connect" (for user ID)
4. Go to Auth tab → OAuth 2.0 tools
5. Select BOTH scopes:
   - w_member_social
   - openid
6. Click "Request access token"
7. Authorize and copy the token

Environment Variables:
  export {get_env_var_name("LINKEDIN_ACCESS_TOKEN")}="..."

Note: Tokens expire after 60 days. Regenerate via OAuth 2.0 tools.

Test:
  socialia post linkedin "Test" --dry-run
""",
        "reddit": f"""
REDDIT SETUP
============

1. Go to https://www.reddit.com/prefs/apps
2. Create 'script' type app

Environment Variables:
  export {get_env_var_name("REDDIT_CLIENT_ID")}="..."
  export {get_env_var_name("REDDIT_CLIENT_SECRET")}="..."
  export {get_env_var_name("REDDIT_USERNAME")}="..."
  export {get_env_var_name("REDDIT_PASSWORD")}="..."

Test:
  socialia post reddit "Test" --subreddit test --dry-run
""",
        "youtube": f"""
YOUTUBE SETUP
=============

1. Go to https://console.cloud.google.com/
2. Enable YouTube Data API v3
3. Create OAuth 2.0 credentials
4. Download client_secrets.json

Environment Variables:
  export {get_env_var_name("YOUTUBE_CLIENT_SECRETS_FILE")}="/path/to/client_secrets.json"

Test:
  socialia post youtube "Test" --video test.mp4 --dry-run
""",
        "analytics": f"""
GOOGLE ANALYTICS SETUP
======================

PART 1: Send Events (Measurement Protocol)
------------------------------------------
1. Go to https://analytics.google.com/
2. Admin > Data Streams > Select your stream
3. Measurement Protocol API secrets > Create

Environment Variables:
  export {get_env_var_name("GOOGLE_ANALYTICS_MEASUREMENT_ID")}="G-XXXXXXXXXX"
  export {get_env_var_name("GOOGLE_ANALYTICS_API_SECRET")}="..."

Test:
  socialia analytics track test_event

PART 2: Read Data (Data API) - Optional
---------------------------------------
1. Get Property ID: GA Admin > Property Settings (numeric, e.g., 379172597)
2. Go to https://console.cloud.google.com/
3. Enable "Google Analytics Data API"
4. IAM & Admin > Service Accounts > Create
   - Name: socialia-analytics
   - Create key > JSON > Download
5. In GA: Admin > Property access management
   - Add service account email with Viewer role

Environment Variables:
  export {get_env_var_name("GOOGLE_ANALYTICS_PROPERTY_ID")}="379172597"
  export GOOGLE_APPLICATION_CREDENTIALS="/path/to/service-account.json"

Test:
  socialia analytics realtime
  socialia analytics pageviews
  socialia analytics sources
""",
    }
    return guides[platform]


def cmd_setup(args) -> int:
    """Show platform setup instructions."""
    platform = getattr(args, "platform", "all") or "all"

    if platform == "all":
        for name in ["twitter", "linkedin", "reddit", "youtube", "analytics"]:
            print(_get_setup_guide(name))
    else:
        print(_get_setup_guide(platform))

    return 0



"""Tests for LinkedIn client."""

from unittest.mock import MagicMock, patch

from socialia.linkedin import LinkedIn


class TestLinkedIn:
    """Test LinkedIn class."""

    def test_init_with_token(self, linkedin_credentials):
        """Test initialization with explicit token."""
        client = LinkedIn(**linkedin_credentials)
        assert client.access_token == "test_linkedin_token"

    def test_init_from_environment(self, monkeypatch):
        """Test initialization from environment variables."""
        # Clear any existing env vars first
        monkeypatch.delenv("SOCIALIA_LINKEDIN_ACCESS_TOKEN", raising=False)
        monkeypatch.delenv("SCITEX_LINKEDIN_ACCESS_TOKEN", raising=False)
        monkeypatch.setenv("SOCIALIA_LINKEDIN_ACCESS_TOKEN", "env_token")
        client = LinkedIn()
        assert client.access_token == "env_token"

    def test_validate_credentials_valid(self, linkedin_credentials):
        """Test credential validation with valid token."""
        client = LinkedIn(**linkedin_credentials)
        assert client.validate_credentials() is True

    def test_validate_credentials_missing(self, monkeypatch):
        """Test credential validation with missing token."""
        # Clear environment variables that might be set
        monkeypatch.delenv("SOCIALIA_LINKEDIN_ACCESS_TOKEN", raising=False)
        monkeypatch.delenv("SCITEX_LINKEDIN_ACCESS_TOKEN", raising=False)
        client = LinkedIn()
        assert client.validate_credentials() is False

    def test_post_missing_credentials(self, monkeypatch):
        """Test post fails with missing credentials."""
        # Clear environment variables that might be set
        monkeypatch.delenv("SOCIALIA_LINKEDIN_ACCESS_TOKEN", raising=False)
        monkeypatch.delenv("SCITEX_LINKEDIN_ACCESS_TOKEN", raising=False)
        client = LinkedIn()
        result = client.post("Test")
        assert result["success"] is False
        assert "token" in result["error"].lower()

    @patch("socialia.linkedin.requests")
    def test_get_user_urn_success(self, mock_requests, linkedin_credentials):
        """Test successful user URN retrieval."""
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"id": "user123"}
        mock_requests.get.return_value = mock_response

        client = LinkedIn(**linkedin_credentials)
        user_urn = client._get_user_urn()

        assert user_urn == "urn:li:person:user123"

    @patch("socialia.linkedin.requests")
    def test_post_success(self, mock_requests, linkedin_credentials):
        """Test successful post."""
        # Mock user URN call
        mock_user_response = MagicMock()
        mock_user_response.status_code = 200
        mock_user_response.json.return_value = {"id": "user123"}

        # Mock post call
        mock_post_response = MagicMock()
        mock_post_response.status_code = 201
        mock_post_response.headers = {"X-RestLi-Id": "share123"}
        mock_post_response.json.return_value = {}

        mock_requests.get.return_value = mock_user_response
        mock_requests.post.return_value = mock_post_response

        client = LinkedIn(**linkedin_credentials)
        result = client.post("Hello LinkedIn!")

        assert result["success"] is True
        assert result["id"] == "share123"

    @patch("socialia.linkedin.requests")
    def test_post_failure(self, mock_requests, linkedin_credentials):
        """Test failed post."""
        # Mock user URN call
        mock_user_response = MagicMock()
        mock_user_response.status_code = 200
        mock_user_response.json.return_value = {"id": "user123"}

        # Mock failed post call
        mock_post_response = MagicMock()
        mock_post_response.status_code = 401
        mock_post_response.text = "Unauthorized"

        mock_requests.get.return_value = mock_user_response
        mock_requests.post.return_value = mock_post_response

        client = LinkedIn(**linkedin_credentials)
        result = client.post("Test post")

        assert result["success"] is False
        assert "401" in result["error"]

    @patch("socialia.linkedin.requests")
    def test_delete_success(self, mock_requests, linkedin_credentials):
        """Test successful delete."""
        mock_response = MagicMock()
        mock_response.status_code = 204

        mock_requests.delete.return_value = mock_response

        client = LinkedIn(**linkedin_credentials)
        result = client.delete("share123")

        assert result["success"] is True
        assert result["deleted"] is True

    @patch("socialia.linkedin.requests")
    def test_delete_failure(self, mock_requests, linkedin_credentials):
        """Test failed delete."""
        mock_response = MagicMock()
        mock_response.status_code = 404
        mock_response.text = "Not Found"

        mock_requests.delete.return_value = mock_response

        client = LinkedIn(**linkedin_credentials)
        result = client.delete("invalid_id")

        assert result["success"] is False
        assert "404" in result["error"]



#!/usr/bin/env python3
"""Branding configuration for white-label integration.

This module provides configurable branding for socialia, allowing parent
packages (e.g., scitex.social) to rebrand and use their own env var prefix.

Environment Variables
---------------------
SOCIALIA_BRAND : str
    Package name shown in docs (default: "socialia")
SOCIALIA_ENV_PREFIX : str
    Environment variable prefix (default: "SOCIALIA")

Usage
-----
Parent package sets env vars before importing:

    # scitex/social/__init__.py
    import os
    os.environ["SOCIALIA_BRAND"] = "scitex.social"
    os.environ["SOCIALIA_ENV_PREFIX"] = "SCITEX"
    from socialia import *

Then env vars like SCITEX_X_CONSUMER_KEY will be used.
"""

import os
from typing import Optional

# Read branding from environment
BRAND_NAME = os.environ.get("SOCIALIA_BRAND", "socialia")
ENV_PREFIX = os.environ.get("SOCIALIA_ENV_PREFIX", "SOCIALIA")

# Original values (for reference/restoration)
_ORIGINAL_NAME = "socialia"
_ORIGINAL_PREFIX = "SOCIALIA"


def get_env(key: str, default: Optional[str] = None) -> Optional[str]:
    """Get environment variable with brand prefix priority.

    Lookup order:
    1. {ENV_PREFIX}_{key} (e.g., SCITEX_X_CONSUMER_KEY when branded)
    2. SOCIALIA_{key} (always as fallback)
    3. {key} (unprefixed)
    4. default

    Parameters
    ----------
    key : str
        Environment variable name without prefix (e.g., "X_CONSUMER_KEY")
    default : str, optional
        Default value if not found

    Returns
    -------
    str or None
        Environment variable value or default
    """
    # Branded prefix (may be SCITEX or custom)
    if ENV_PREFIX != _ORIGINAL_PREFIX:
        value = os.environ.get(f"{ENV_PREFIX}_{key}")
        if value:
            return value

    # Always try SOCIALIA_ as fallback
    value = os.environ.get(f"SOCIALIA_{key}")
    if value:
        return value

    # Unprefixed
    value = os.environ.get(key)
    if value:
        return value

    return default


def get_env_var_name(key: str) -> str:
    """Get the branded environment variable name for documentation.

    Parameters
    ----------
    key : str
        Variable name without prefix

    Returns
    -------
    str
        Full variable name with brand prefix
    """
    return f"{ENV_PREFIX}_{key}"


def get_mcp_server_name() -> str:
    """Get the MCP server name based on branding.

    Returns
    -------
    str
        Server name for MCP registration.
    """
    return BRAND_NAME.replace(".", "-")


# EOF



"""Socialia - Unified social media management: posting, analytics, and insights."""

__version__ = "0.1.2"

from .base import BasePoster
from .twitter import Twitter
from .linkedin import LinkedIn
from .reddit import Reddit
from .analytics import GoogleAnalytics
from .youtube import YouTube

# Backward compatibility aliases
TwitterPoster = Twitter
LinkedInPoster = LinkedIn
RedditPoster = Reddit
YouTubePoster = YouTube

__all__ = [
    # Base
    "BasePoster",
    # Platform clients (new names)
    "Twitter",
    "LinkedIn",
    "Reddit",
    "YouTube",
    "GoogleAnalytics",
    # Backward compatibility aliases
    "TwitterPoster",
    "LinkedInPoster",
    "RedditPoster",
    "YouTubePoster",
    # Version
    "__version__",
]



#!/usr/bin/env python3
"""
Socialia CLI - Unified social media management from command line.

Usage:
    socialia post twitter "Hello World!"
    socialia delete twitter 1234567890
    socialia thread twitter --file thread.txt
    socialia status
    socialia mcp run
    socialia --help
"""

import argparse
import sys
from pathlib import Path

from .. import __version__
from ._commands import (
    cmd_post,
    cmd_delete,
    cmd_thread,
    cmd_analytics,
    cmd_status,
    cmd_mcp,
    cmd_setup,
)

PLATFORMS = ["twitter", "linkedin", "reddit", "youtube"]


def _get_epilog() -> str:
    """Generate epilog with branded env var names."""
    from .._branding import get_env_var_name

    return f"""
Examples:
  socialia post twitter "Hello World!"
  socialia post twitter --file tweet.txt
  socialia delete twitter 1234567890
  socialia thread twitter --file thread.txt
  socialia status
  socialia mcp run
  socialia --help-recursive

Environment Variables:
  {get_env_var_name("X_CONSUMER_KEY"):36} Twitter API consumer key
  {get_env_var_name("X_CONSUMER_KEY_SECRET"):36} Twitter API consumer secret
  {get_env_var_name("X_ACCESSTOKEN"):36} Twitter access token
  {get_env_var_name("X_ACCESSTOKEN_SECRET"):36} Twitter access token secret
  {get_env_var_name("LINKEDIN_ACCESS_TOKEN"):36} LinkedIn OAuth access token
  {get_env_var_name("REDDIT_CLIENT_ID"):36} Reddit app client ID
  {get_env_var_name("REDDIT_CLIENT_SECRET"):36} Reddit app client secret
  {get_env_var_name("REDDIT_USERNAME"):36} Reddit username
  {get_env_var_name("REDDIT_PASSWORD"):36} Reddit password
  {get_env_var_name("GOOGLE_ANALYTICS_MEASUREMENT_ID"):36} Google Analytics measurement ID
  {get_env_var_name("GOOGLE_ANALYTICS_API_SECRET"):36} Google Analytics API secret
"""


def create_parser() -> argparse.ArgumentParser:
    """Create the argument parser with all subcommands."""
    parser = argparse.ArgumentParser(
        prog="socialia",
        description="Unified social media management CLI",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=_get_epilog(),
    )
    parser.add_argument(
        "-v", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    parser.add_argument("--json", action="store_true", help="Output results as JSON")
    parser.add_argument(
        "--help-recursive", action="store_true", help="Show help for all commands"
    )

    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # post command
    post_parser = subparsers.add_parser(
        "post",
        help="Post content to a platform",
        description="Post content to a social media platform",
    )
    post_parser.add_argument("platform", choices=PLATFORMS, help="Target platform")
    post_parser.add_argument("text", nargs="?", help="Content to post")
    post_parser.add_argument("-f", "--file", type=Path, help="Read content from file")
    post_parser.add_argument("--reply-to", help="Post ID to reply to (Twitter)")
    post_parser.add_argument("--quote", help="Post ID to quote (Twitter)")
    post_parser.add_argument(
        "--subreddit", "-s", default="test", help="Target subreddit (Reddit)"
    )
    post_parser.add_argument("--title", "-t", help="Post title (Reddit/YouTube)")
    post_parser.add_argument(
        "--video", "-V", type=Path, help="Video file path (YouTube)"
    )
    post_parser.add_argument("--thumbnail", type=Path, help="Thumbnail image (YouTube)")
    post_parser.add_argument("--tags", help="Comma-separated tags (YouTube)")
    post_parser.add_argument(
        "--privacy",
        choices=["public", "private", "unlisted"],
        default="public",
        help="Privacy status (YouTube)",
    )
    post_parser.add_argument(
        "-n", "--dry-run", action="store_true", help="Print without posting"
    )

    # delete command
    delete_parser = subparsers.add_parser(
        "delete", help="Delete a post", description="Delete a post by ID"
    )
    delete_parser.add_argument("platform", choices=PLATFORMS, help="Target platform")
    delete_parser.add_argument("post_id", help="Post ID to delete")

    # thread command
    thread_parser = subparsers.add_parser(
        "thread", help="Post a thread", description="Post a thread of connected posts"
    )
    thread_parser.add_argument("platform", choices=PLATFORMS, help="Target platform")
    thread_parser.add_argument(
        "-f",
        "--file",
        type=Path,
        required=True,
        help="File with thread content (separate posts with ---)",
    )
    thread_parser.add_argument(
        "-n", "--dry-run", action="store_true", help="Print without posting"
    )

    # analytics command
    analytics_parser = subparsers.add_parser(
        "analytics",
        help="Google Analytics operations",
        description="Track events and retrieve analytics data",
    )
    analytics_sub = analytics_parser.add_subparsers(
        dest="analytics_command", help="Analytics operations"
    )

    track_parser = analytics_sub.add_parser(
        "track",
        help="Track a custom event",
        description="Send event to Google Analytics",
    )
    track_parser.add_argument("event_name", help="Event name")
    track_parser.add_argument(
        "--param",
        "-p",
        action="append",
        nargs=2,
        metavar=("KEY", "VALUE"),
        help="Event parameter (can be repeated)",
    )

    analytics_sub.add_parser(
        "realtime",
        help="Get realtime active users",
        description="Query realtime user count",
    )

    pageviews_parser = analytics_sub.add_parser(
        "pageviews", help="Get page view metrics", description="Query page view data"
    )
    pageviews_parser.add_argument("--start", default="7daysAgo", help="Start date")
    pageviews_parser.add_argument("--end", default="today", help="End date")
    pageviews_parser.add_argument("--path", help="Filter by page path")

    sources_parser = analytics_sub.add_parser(
        "sources",
        help="Get traffic sources",
        description="Query traffic source breakdown",
    )
    sources_parser.add_argument("--start", default="7daysAgo", help="Start date")
    sources_parser.add_argument("--end", default="today", help="End date")

    # status command
    subparsers.add_parser(
        "status",
        help="Show configuration and environment status",
        description="Display current configuration and environment variable status",
    )

    # mcp command
    mcp_parser = subparsers.add_parser(
        "mcp",
        help="MCP server commands",
        description="Model Context Protocol server operations",
    )
    mcp_sub = mcp_parser.add_subparsers(dest="mcp_command", help="MCP operations")
    mcp_sub.add_parser(
        "run", help="Run the MCP server", description="Start the MCP server"
    )
    mcp_sub.add_parser(
        "info",
        help="Show MCP server info",
        description="Display MCP server information",
    )

    # setup command
    setup_parser = subparsers.add_parser(
        "setup",
        help="Show platform setup instructions",
        description="Display API setup guide for platforms",
    )
    setup_parser.add_argument(
        "platform",
        nargs="?",
        choices=["twitter", "linkedin", "reddit", "youtube", "analytics", "all"],
        default="all",
        help="Platform to show setup for (default: all)",
    )

    return parser


def cmd_help_recursive(parser: argparse.ArgumentParser) -> int:
    """Show help for all commands."""
    print("=" * 60)
    print("SOCIALIA - Complete Command Reference")
    print("=" * 60)
    print()

    parser.print_help()
    print()

    for action in parser._subparsers._actions:
        if isinstance(action, argparse._SubParsersAction):
            for name, subparser in action.choices.items():
                print("-" * 60)
                print(f"Command: {name}")
                print("-" * 60)
                subparser.print_help()
                print()

    return 0


def main(argv: list[str] = None) -> int:
    """Main entry point."""
    parser = create_parser()
    args = parser.parse_args(argv)

    if args.help_recursive:
        return cmd_help_recursive(parser)

    if args.command is None:
        parser.print_help()
        return 0

    if args.command == "post":
        return cmd_post(args, output_json=args.json)
    elif args.command == "delete":
        return cmd_delete(args, output_json=args.json)
    elif args.command == "thread":
        return cmd_thread(args, output_json=args.json)
    elif args.command == "analytics":
        return cmd_analytics(args, output_json=args.json)
    elif args.command == "status":
        return cmd_status(output_json=args.json)
    elif args.command == "mcp":
        return cmd_mcp(args)
    elif args.command == "setup":
        return cmd_setup(args)
    else:
        parser.print_help()
        return 1


if __name__ == "__main__":
    sys.exit(main())



"""Google Analytics integration for tracking and metrics."""

import os
import requests
from typing import Optional
from datetime import datetime

from ._branding import get_env


class GoogleAnalytics:
    """
    Google Analytics 4 integration.

    Supports:
    - Measurement Protocol (sending events)
    - Data API (retrieving metrics) - requires service account

    Environment Variables (use branded prefix, e.g., SOCIALIA_):
        GOOGLE_ANALYTICS_MEASUREMENT_ID: GA4 Measurement ID (G-XXXXXXXXXX)
        GOOGLE_ANALYTICS_API_SECRET: Measurement Protocol API secret
        GOOGLE_ANALYTICS_PROPERTY_ID: Property ID (numeric, for Data API)
        GOOGLE_APPLICATION_CREDENTIALS: Path to service account JSON (for Data API)
    """

    def __init__(
        self,
        measurement_id: Optional[str] = None,
        api_secret: Optional[str] = None,
        property_id: Optional[str] = None,
    ):
        """
        Initialize Google Analytics client.

        Args:
            measurement_id: GA4 Measurement ID (G-XXXXXXXXXX)
            api_secret: Measurement Protocol API secret
            property_id: GA4 Property ID (numeric, for Data API)
        """
        # Google Analytics credentials (use branding-aware get_env)
        self.measurement_id = measurement_id or (
            get_env("GOOGLE_ANALYTICS_MEASUREMENT_ID") or get_env("GA_MEASUREMENT_ID")
        )
        self.api_secret = api_secret or (
            get_env("GOOGLE_ANALYTICS_API_SECRET") or get_env("GA_API_SECRET")
        )
        self.property_id = property_id or (
            get_env("GOOGLE_ANALYTICS_PROPERTY_ID") or get_env("GA_PROPERTY_ID")
        )

        # Google Application Credentials (for Data API service account)
        credentials_path = get_env("GOOGLE_APPLICATION_CREDENTIALS") or get_env(
            "GA_CREDENTIALS"
        )
        if credentials_path and not os.environ.get("GOOGLE_APPLICATION_CREDENTIALS"):
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = os.path.expandvars(
                credentials_path
            )

        # Measurement Protocol endpoint
        self.mp_endpoint = "https://www.google-analytics.com/mp/collect"

    def validate_credentials(self) -> dict:
        """Check which features are available based on credentials."""
        return {
            "measurement_protocol": bool(self.measurement_id and self.api_secret),
            "data_api": bool(self.property_id),
        }

    def track_event(
        self,
        name: str,
        params: Optional[dict] = None,
        client_id: Optional[str] = None,
        user_id: Optional[str] = None,
    ) -> dict:
        """
        Send event to Google Analytics via Measurement Protocol.

        Args:
            name: Event name (e.g., 'social_post', 'social_share')
            params: Event parameters dict
            client_id: Client ID (generated if not provided)
            user_id: Optional user ID for cross-device tracking

        Returns:
            dict with 'success' and details

        Example:
            ga.track_event('social_post', {
                'platform': 'twitter',
                'post_id': '123456',
                'content_length': 280,
            })
        """
        if not self.measurement_id or not self.api_secret:
            return {
                "success": False,
                "error": "Missing GA_MEASUREMENT_ID or GA_API_SECRET",
            }

        # Generate client_id if not provided
        if not client_id:
            import uuid

            client_id = str(uuid.uuid4())

        # Build payload
        payload = {
            "client_id": client_id,
            "events": [
                {
                    "name": name,
                    "params": params or {},
                }
            ],
        }

        if user_id:
            payload["user_id"] = user_id

        # Add timestamp
        payload["events"][0]["params"]["engagement_time_msec"] = "100"

        try:
            url = f"{self.mp_endpoint}?measurement_id={self.measurement_id}&api_secret={self.api_secret}"
            response = requests.post(url, json=payload, timeout=10)

            # Measurement Protocol returns 204 on success (no content)
            if response.status_code in (200, 204):
                return {
                    "success": True,
                    "client_id": client_id,
                    "event": name,
                }
            else:
                return {
                    "success": False,
                    "error": f"HTTP {response.status_code}: {response.text}",
                }

        except Exception as e:
            return {"success": False, "error": str(e)}

    def track_social_post(
        self,
        platform: str,
        post_id: str,
        content_type: str = "text",
        success: bool = True,
    ) -> dict:
        """
        Track a social media post event.

        Args:
            platform: Platform name (twitter, linkedin, reddit)
            post_id: Post ID from the platform
            content_type: Type of content (text, link, image, thread)
            success: Whether the post was successful

        Returns:
            dict with tracking result
        """
        return self.track_event(
            "social_post",
            {
                "platform": platform,
                "post_id": post_id,
                "content_type": content_type,
                "success": str(success).lower(),
                "timestamp": datetime.utcnow().isoformat(),
            },
        )

    def track_social_delete(self, platform: str, post_id: str) -> dict:
        """Track a social media delete event."""
        return self.track_event(
            "social_delete",
            {
                "platform": platform,
                "post_id": post_id,
                "timestamp": datetime.utcnow().isoformat(),
            },
        )

    def get_realtime_users(self) -> dict:
        """
        Get realtime active users (requires Data API setup).

        Returns:
            dict with 'success' and 'active_users' count
        """
        if not self.property_id:
            return {
                "success": False,
                "error": "Missing GA_PROPERTY_ID for Data API",
            }

        try:
            from google.analytics.data_v1beta import BetaAnalyticsDataClient
            from google.analytics.data_v1beta.types import (
                RunRealtimeReportRequest,
                Metric,
            )

            client = BetaAnalyticsDataClient()
            request = RunRealtimeReportRequest(
                property=f"properties/{self.property_id}",
                metrics=[Metric(name="activeUsers")],
            )
            response = client.run_realtime_report(request)

            active_users = 0
            if response.rows:
                active_users = int(response.rows[0].metric_values[0].value)

            return {
                "success": True,
                "active_users": active_users,
            }

        except ImportError:
            return {
                "success": False,
                "error": "google-analytics-data not installed. Run: pip install google-analytics-data",
            }
        except Exception as e:
            return {"success": False, "error": str(e)}

    def get_page_views(
        self,
        start_date: str = "7daysAgo",
        end_date: str = "today",
        page_path: Optional[str] = None,
    ) -> dict:
        """
        Get page view metrics (requires Data API setup).

        Args:
            start_date: Start date (YYYY-MM-DD or relative like '7daysAgo')
            end_date: End date (YYYY-MM-DD or 'today')
            page_path: Optional page path filter

        Returns:
            dict with 'success' and metrics
        """
        if not self.property_id:
            return {
                "success": False,
                "error": "Missing GA_PROPERTY_ID for Data API",
            }

        try:
            from google.analytics.data_v1beta import BetaAnalyticsDataClient
            from google.analytics.data_v1beta.types import (
                RunReportRequest,
                DateRange,
                Metric,
                Dimension,
                FilterExpression,
                Filter,
            )

            client = BetaAnalyticsDataClient()

            request_params = {
                "property": f"properties/{self.property_id}",
                "date_ranges": [DateRange(start_date=start_date, end_date=end_date)],
                "metrics": [
                    Metric(name="screenPageViews"),
                    Metric(name="sessions"),
                    Metric(name="totalUsers"),
                ],
                "dimensions": [Dimension(name="pagePath")],
            }

            if page_path:
                request_params["dimension_filter"] = FilterExpression(
                    filter=Filter(
                        field_name="pagePath",
                        string_filter=Filter.StringFilter(
                            match_type=Filter.StringFilter.MatchType.CONTAINS,
                            value=page_path,
                        ),
                    )
                )

            response = client.run_report(RunReportRequest(**request_params))

            pages = []
            for row in response.rows:
                pages.append(
                    {
                        "path": row.dimension_values[0].value,
                        "page_views": int(row.metric_values[0].value),
                        "sessions": int(row.metric_values[1].value),
                        "users": int(row.metric_values[2].value),
                    }
                )

            return {
                "success": True,
                "date_range": f"{start_date} to {end_date}",
                "pages": pages,
            }

        except ImportError:
            return {
                "success": False,
                "error": "google-analytics-data not installed. Run: pip install google-analytics-data",
            }
        except Exception as e:
            return {"success": False, "error": str(e)}

    def get_traffic_sources(
        self,
        start_date: str = "7daysAgo",
        end_date: str = "today",
    ) -> dict:
        """
        Get traffic source breakdown (requires Data API setup).

        Returns:
            dict with traffic sources and their metrics
        """
        if not self.property_id:
            return {
                "success": False,
                "error": "Missing GA_PROPERTY_ID for Data API",
            }

        try:
            from google.analytics.data_v1beta import BetaAnalyticsDataClient
            from google.analytics.data_v1beta.types import (
                RunReportRequest,
                DateRange,
                Metric,
                Dimension,
            )

            client = BetaAnalyticsDataClient()
            response = client.run_report(
                RunReportRequest(
                    property=f"properties/{self.property_id}",
                    date_ranges=[DateRange(start_date=start_date, end_date=end_date)],
                    metrics=[
                        Metric(name="sessions"),
                        Metric(name="totalUsers"),
                    ],
                    dimensions=[
                        Dimension(name="sessionSource"),
                        Dimension(name="sessionMedium"),
                    ],
                )
            )

            sources = []
            for row in response.rows:
                sources.append(
                    {
                        "source": row.dimension_values[0].value,
                        "medium": row.dimension_values[1].value,
                        "sessions": int(row.metric_values[0].value),
                        "users": int(row.metric_values[1].value),
                    }
                )

            # Sort by sessions descending
            sources.sort(key=lambda x: x["sessions"], reverse=True)

            return {
                "success": True,
                "date_range": f"{start_date} to {end_date}",
                "sources": sources,
            }

        except ImportError:
            return {
                "success": False,
                "error": "google-analytics-data not installed. Run: pip install google-analytics-data",
            }
        except Exception as e:
            return {"success": False, "error": str(e)}



"""YouTube API integration for video uploads and community posts."""

import os
from typing import Optional

from .base import BasePoster
from ._branding import get_env

try:
    from google.oauth2.credentials import Credentials
    from google_auth_oauthlib.flow import InstalledAppFlow
    from google.auth.transport.requests import Request
    from googleapiclient.discovery import build
    from googleapiclient.http import MediaFileUpload

    HAS_YOUTUBE = True
except ImportError:
    HAS_YOUTUBE = False


# OAuth scopes for YouTube
SCOPES = [
    "https://www.googleapis.com/auth/youtube.upload",
    "https://www.googleapis.com/auth/youtube",
    "https://www.googleapis.com/auth/youtube.force-ssl",
]


class YouTube(BasePoster):
    """
    YouTube API client for video uploads and management.

    Supports:
    - Video uploads with metadata
    - Update video metadata
    - Delete videos
    - Community posts (if channel eligible)

    Environment Variables:
        YOUTUBE_CLIENT_SECRETS_FILE: Path to OAuth client secrets JSON
        YOUTUBE_TOKEN_FILE: Path to store OAuth tokens (default: ~/.youtube_token.json)
    """

    def __init__(
        self,
        client_secrets_file: Optional[str] = None,
        token_file: Optional[str] = None,
    ):
        """
        Initialize YouTube client.

        Args:
            client_secrets_file: Path to OAuth client secrets JSON from Google Console
            token_file: Path to store/load OAuth tokens
        """
        self.client_secrets_file = client_secrets_file or get_env(
            "YOUTUBE_CLIENT_SECRETS_FILE"
        )
        self.token_file = token_file or (
            get_env("YOUTUBE_TOKEN_FILE") or os.path.expanduser("~/.youtube_token.json")
        )
        self._youtube = None
        self._credentials = None

    def validate_credentials(self) -> bool:
        """Check if credentials are available."""
        if not HAS_YOUTUBE:
            return False

        # Check for existing token
        if os.path.exists(self.token_file):
            return True

        # Check for client secrets to initiate OAuth
        if self.client_secrets_file and os.path.exists(self.client_secrets_file):
            return True

        return False

    def _get_credentials(self) -> Optional["Credentials"]:
        """Get or refresh OAuth credentials."""
        if not HAS_YOUTUBE:
            return None

        creds = None

        # Load existing token
        if os.path.exists(self.token_file):
            creds = Credentials.from_authorized_user_file(self.token_file, SCOPES)

        # Refresh or get new credentials
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            elif self.client_secrets_file and os.path.exists(self.client_secrets_file):
                flow = InstalledAppFlow.from_client_secrets_file(
                    self.client_secrets_file, SCOPES
                )
                creds = flow.run_local_server(port=0)
            else:
                return None

            # Save credentials
            with open(self.token_file, "w") as f:
                f.write(creds.to_json())

        return creds

    def _get_client(self):
        """Get authenticated YouTube client."""
        if self._youtube:
            return self._youtube

        creds = self._get_credentials()
        if not creds:
            return None

        self._youtube = build("youtube", "v3", credentials=creds)
        return self._youtube

    def post(
        self,
        text: str,
        video_path: Optional[str] = None,
        title: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[list] = None,
        category_id: str = "22",  # People & Blogs
        privacy_status: str = "public",
        thumbnail_path: Optional[str] = None,
    ) -> dict:
        """
        Upload a video or create a community post.

        Args:
            text: Description text (or community post text if no video)
            video_path: Path to video file (required for video upload)
            title: Video title (uses first line of text if not provided)
            description: Video description (uses text if not provided)
            tags: List of tags for the video
            category_id: YouTube category ID (default: 22 = People & Blogs)
            privacy_status: 'public', 'private', or 'unlisted'
            thumbnail_path: Path to custom thumbnail image

        Returns:
            dict with 'success', 'id', 'url' or 'error'
        """
        if not HAS_YOUTUBE:
            return {
                "success": False,
                "error": "YouTube libraries not installed. Run: pip install google-api-python-client google-auth-oauthlib",
            }

        if not self.validate_credentials():
            return {"success": False, "error": "Missing YouTube credentials"}

        youtube = self._get_client()
        if not youtube:
            return {"success": False, "error": "Could not create YouTube client"}

        # Video upload
        if video_path:
            return self._upload_video(
                youtube,
                video_path=video_path,
                title=title or text.split("\n")[0][:100],
                description=description or text,
                tags=tags or [],
                category_id=category_id,
                privacy_status=privacy_status,
                thumbnail_path=thumbnail_path,
            )

        # Community post (text only)
        return self._create_community_post(youtube, text)

    def _upload_video(
        self,
        youtube,
        video_path: str,
        title: str,
        description: str,
        tags: list,
        category_id: str,
        privacy_status: str,
        thumbnail_path: Optional[str] = None,
    ) -> dict:
        """Upload a video to YouTube."""
        if not os.path.exists(video_path):
            return {"success": False, "error": f"Video file not found: {video_path}"}

        body = {
            "snippet": {
                "title": title,
                "description": description,
                "tags": tags,
                "categoryId": category_id,
            },
            "status": {
                "privacyStatus": privacy_status,
                "selfDeclaredMadeForKids": False,
            },
        }

        try:
            media = MediaFileUpload(
                video_path,
                chunksize=1024 * 1024,  # 1MB chunks
                resumable=True,
            )

            request = youtube.videos().insert(
                part=",".join(body.keys()),
                body=body,
                media_body=media,
            )

            response = None
            while response is None:
                status, response = request.next_chunk()

            video_id = response["id"]

            # Set custom thumbnail if provided
            if thumbnail_path and os.path.exists(thumbnail_path):
                try:
                    youtube.thumbnails().set(
                        videoId=video_id,
                        media_body=MediaFileUpload(thumbnail_path),
                    ).execute()
                except Exception:
                    pass  # Thumbnail upload is optional

            return {
                "success": True,
                "id": video_id,
                "url": f"https://www.youtube.com/watch?v={video_id}",
                "title": title,
            }

        except Exception as e:
            return {"success": False, "error": str(e)}

    def _create_community_post(self, youtube, text: str) -> dict:
        """
        Create a community post.

        Note: Community posts require channel to be eligible (usually 500+ subscribers).
        This uses the YouTube Data API's activities endpoint.
        """
        try:
            # Community posts via API have limited support
            # This is a placeholder - full implementation requires channel eligibility
            return {
                "success": False,
                "error": "Community posts require channel eligibility (500+ subscribers) and manual posting via YouTube Studio",
            }
        except Exception as e:
            return {"success": False, "error": str(e)}

    def delete(self, video_id: str) -> dict:
        """
        Delete a video.

        Args:
            video_id: YouTube video ID

        Returns:
            dict with 'success' or 'error'
        """
        if not HAS_YOUTUBE:
            return {"success": False, "error": "YouTube libraries not installed"}

        if not self.validate_credentials():
            return {"success": False, "error": "Missing credentials"}

        youtube = self._get_client()
        if not youtube:
            return {"success": False, "error": "Could not create YouTube client"}

        try:
            youtube.videos().delete(id=video_id).execute()
            return {"success": True, "deleted": True}
        except Exception as e:
            return {"success": False, "error": str(e)}

    def update(
        self,
        video_id: str,
        title: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[list] = None,
        privacy_status: Optional[str] = None,
    ) -> dict:
        """
        Update video metadata.

        Args:
            video_id: YouTube video ID
            title: New title (optional)
            description: New description (optional)
            tags: New tags (optional)
            privacy_status: New privacy status (optional)

        Returns:
            dict with 'success' or 'error'
        """
        if not HAS_YOUTUBE:
            return {"success": False, "error": "YouTube libraries not installed"}

        youtube = self._get_client()
        if not youtube:
            return {"success": False, "error": "Could not create YouTube client"}

        try:
            # Get current video details
            current = (
                youtube.videos()
                .list(
                    part="snippet,status",
                    id=video_id,
                )
                .execute()
            )

            if not current.get("items"):
                return {"success": False, "error": f"Video not found: {video_id}"}

            video = current["items"][0]
            snippet = video["snippet"]
            status = video["status"]

            # Update fields
            if title:
                snippet["title"] = title
            if description:
                snippet["description"] = description
            if tags:
                snippet["tags"] = tags
            if privacy_status:
                status["privacyStatus"] = privacy_status

            # Update video
            youtube.videos().update(
                part="snippet,status",
                body={
                    "id": video_id,
                    "snippet": snippet,
                    "status": status,
                },
            ).execute()

            return {
                "success": True,
                "id": video_id,
                "url": f"https://www.youtube.com/watch?v={video_id}",
            }

        except Exception as e:
            return {"success": False, "error": str(e)}

    def get_channel_info(self) -> dict:
        """
        Get authenticated user's channel information.

        Returns:
            dict with channel info or error
        """
        if not HAS_YOUTUBE:
            return {"success": False, "error": "YouTube libraries not installed"}

        youtube = self._get_client()
        if not youtube:
            return {"success": False, "error": "Could not create YouTube client"}

        try:
            response = (
                youtube.channels()
                .list(
                    part="snippet,statistics",
                    mine=True,
                )
                .execute()
            )

            if not response.get("items"):
                return {"success": False, "error": "No channel found"}

            channel = response["items"][0]
            return {
                "success": True,
                "id": channel["id"],
                "title": channel["snippet"]["title"],
                "description": channel["snippet"].get("description", ""),
                "subscribers": channel["statistics"].get("subscriberCount", "hidden"),
                "videos": channel["statistics"].get("videoCount", 0),
                "views": channel["statistics"].get("viewCount", 0),
                "url": f"https://www.youtube.com/channel/{channel['id']}",
            }

        except Exception as e:
            return {"success": False, "error": str(e)}

    def list_videos(self, max_results: int = 10) -> dict:
        """
        List user's uploaded videos.

        Args:
            max_results: Maximum number of videos to return

        Returns:
            dict with videos list or error
        """
        if not HAS_YOUTUBE:
            return {"success": False, "error": "YouTube libraries not installed"}

        youtube = self._get_client()
        if not youtube:
            return {"success": False, "error": "Could not create YouTube client"}

        try:
            # Get uploads playlist ID
            channels = (
                youtube.channels()
                .list(
                    part="contentDetails",
                    mine=True,
                )
                .execute()
            )

            if not channels.get("items"):
                return {"success": False, "error": "No channel found"}

            uploads_id = channels["items"][0]["contentDetails"]["relatedPlaylists"][
                "uploads"
            ]

            # Get videos from uploads playlist
            videos_response = (
                youtube.playlistItems()
                .list(
                    part="snippet",
                    playlistId=uploads_id,
                    maxResults=max_results,
                )
                .execute()
            )

            videos = []
            for item in videos_response.get("items", []):
                snippet = item["snippet"]
                video_id = snippet["resourceId"]["videoId"]
                videos.append(
                    {
                        "id": video_id,
                        "title": snippet["title"],
                        "description": snippet.get("description", "")[:100],
                        "published_at": snippet["publishedAt"],
                        "url": f"https://www.youtube.com/watch?v={video_id}",
                    }
                )

            return {
                "success": True,
                "videos": videos,
                "count": len(videos),
            }

        except Exception as e:
            return {"success": False, "error": str(e)}



"""Tests for socialia CLI."""

import pytest
from socialia.cli import create_parser, main


class TestCLIParser:
    """Test CLI argument parsing."""

    def test_parser_creation(self):
        """Test parser is created successfully."""
        parser = create_parser()
        assert parser is not None
        assert parser.prog == "socialia"

    def test_version_argument(self, capsys):
        """Test --version argument."""
        parser = create_parser()
        with pytest.raises(SystemExit) as exc_info:
            parser.parse_args(["--version"])
        assert exc_info.value.code == 0

    def test_post_command_parsing(self):
        """Test post command parsing."""
        parser = create_parser()
        args = parser.parse_args(["post", "twitter", "Hello World"])
        assert args.command == "post"
        assert args.platform == "twitter"
        assert args.text == "Hello World"

    def test_post_dry_run(self):
        """Test post dry-run flag."""
        parser = create_parser()
        args = parser.parse_args(["post", "twitter", "Test", "--dry-run"])
        assert args.dry_run is True

    def test_delete_command_parsing(self):
        """Test delete command parsing."""
        parser = create_parser()
        args = parser.parse_args(["delete", "twitter", "123456"])
        assert args.command == "delete"
        assert args.platform == "twitter"
        assert args.post_id == "123456"

    def test_analytics_track_parsing(self):
        """Test analytics track command parsing."""
        parser = create_parser()
        args = parser.parse_args(["analytics", "track", "test_event"])
        assert args.command == "analytics"
        assert args.analytics_command == "track"
        assert args.event_name == "test_event"


class TestCLIDryRun:
    """Test CLI dry-run functionality."""

    def test_post_dry_run_twitter(self, capsys):
        """Test dry-run post to Twitter."""
        result = main(["post", "twitter", "Test message", "--dry-run"])
        assert result == 0
        captured = capsys.readouterr()
        assert "DRY RUN" in captured.out
        assert "twitter" in captured.out.lower()

    def test_post_dry_run_linkedin(self, capsys):
        """Test dry-run post to LinkedIn."""
        result = main(["post", "linkedin", "Test message", "--dry-run"])
        assert result == 0
        captured = capsys.readouterr()
        assert "DRY RUN" in captured.out
        assert "linkedin" in captured.out.lower()


class TestCLIMain:
    """Test CLI main function."""

    def test_no_command_shows_help(self, capsys):
        """Test that no command shows help."""
        result = main([])
        assert result == 0
        captured = capsys.readouterr()
        assert "usage:" in captured.out.lower() or "socialia" in captured.out.lower()

    def test_help_recursive(self, capsys):
        """Test --help-recursive option."""
        result = main(["--help-recursive"])
        assert result == 0
        captured = capsys.readouterr()
        assert "SOCIALIA" in captured.out
        assert "post" in captured.out.lower()
        assert "delete" in captured.out.lower()

    def test_post_missing_text_error(self, capsys):
        """Test post without text shows error."""
        result = main(["post", "twitter"])
        assert result == 1
        captured = capsys.readouterr()
        assert "error" in captured.err.lower()

    def test_status_command(self, capsys):
        """Test status command."""
        result = main(["status"])
        assert result == 0
        captured = capsys.readouterr()
        assert "Socialia" in captured.out
        assert "TWITTER" in captured.out or "twitter" in captured.out.lower()

    def test_mcp_info_command(self, capsys):
        """Test mcp info command."""
        result = main(["mcp", "info"])
        assert result == 0
        captured = capsys.readouterr()
        assert "MCP" in captured.out
        assert "social_post" in captured.out



"""Reddit API client using PRAW."""

from typing import Optional

from ._branding import get_env
from .base import BasePoster

try:
    import praw

    HAS_PRAW = True
except ImportError:
    HAS_PRAW = False


class Reddit(BasePoster):
    """Reddit API client using PRAW (Python Reddit API Wrapper)."""

    def __init__(
        self,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        user_agent: Optional[str] = None,
    ):
        self.client_id = client_id or get_env("REDDIT_CLIENT_ID")
        self.client_secret = client_secret or get_env("REDDIT_CLIENT_SECRET")
        self.username = username or get_env("REDDIT_USERNAME")
        self.password = password or get_env("REDDIT_PASSWORD")
        self.user_agent = user_agent or (
            get_env("REDDIT_USER_AGENT") or "Socialia v0.1"
        )
        self._reddit: Optional["praw.Reddit"] = None

    def _get_client(self) -> Optional["praw.Reddit"]:
        """Get authenticated Reddit client."""
        if not HAS_PRAW:
            return None

        if self._reddit:
            return self._reddit

        if not self.validate_credentials():
            return None

        self._reddit = praw.Reddit(
            client_id=self.client_id,
            client_secret=self.client_secret,
            username=self.username,
            password=self.password,
            user_agent=self.user_agent,
        )
        return self._reddit

    def validate_credentials(self) -> bool:
        """Check if all credentials are set."""
        if not HAS_PRAW:
            return False
        return all(
            [
                self.client_id,
                self.client_secret,
                self.username,
                self.password,
            ]
        )

    def post(
        self,
        text: str,
        subreddit: str = "test",
        title: Optional[str] = None,
        url: Optional[str] = None,
        flair_id: Optional[str] = None,
    ) -> dict:
        """
        Post to a subreddit.

        Args:
            text: Post body (for self posts) or ignored for link posts
            subreddit: Target subreddit name (without r/)
            title: Post title (required for Reddit)
            url: URL for link posts (makes it a link post instead of self post)
            flair_id: Optional flair ID

        Returns:
            dict with 'success', 'id', 'url' or 'error'
        """
        if not HAS_PRAW:
            return {
                "success": False,
                "error": "PRAW not installed. Run: pip install praw",
            }

        if not self.validate_credentials():
            return {"success": False, "error": "Missing credentials"}

        if not title:
            # Use first line or truncated text as title
            title = text.split("\n")[0][:300]

        reddit = self._get_client()
        if not reddit:
            return {"success": False, "error": "Could not create Reddit client"}

        try:
            sub = reddit.subreddit(subreddit)

            if url:
                # Link post
                submission = sub.submit(
                    title=title,
                    url=url,
                    flair_id=flair_id,
                )
            else:
                # Self/text post
                submission = sub.submit(
                    title=title,
                    selftext=text,
                    flair_id=flair_id,
                )

            return {
                "success": True,
                "id": submission.id,
                "url": f"https://reddit.com{submission.permalink}",
            }

        except praw.exceptions.APIException as e:
            return {"success": False, "error": f"Reddit API error: {e}"}
        except Exception as e:
            return {"success": False, "error": str(e)}

    def delete(self, post_id: str) -> dict:
        """
        Delete a Reddit post.

        Args:
            post_id: Reddit submission ID

        Returns:
            dict with 'success' or 'error'
        """
        if not HAS_PRAW:
            return {"success": False, "error": "PRAW not installed"}

        if not self.validate_credentials():
            return {"success": False, "error": "Missing credentials"}

        reddit = self._get_client()
        if not reddit:
            return {"success": False, "error": "Could not create Reddit client"}

        try:
            submission = reddit.submission(id=post_id)
            submission.delete()
            return {"success": True, "deleted": True}
        except Exception as e:
            return {"success": False, "error": str(e)}

    def comment(
        self,
        post_id: str,
        text: str,
    ) -> dict:
        """
        Add a comment to a post.

        Args:
            post_id: Reddit submission ID
            text: Comment text

        Returns:
            dict with 'success', 'id' or 'error'
        """
        if not HAS_PRAW:
            return {"success": False, "error": "PRAW not installed"}

        if not self.validate_credentials():
            return {"success": False, "error": "Missing credentials"}

        reddit = self._get_client()
        if not reddit:
            return {"success": False, "error": "Could not create Reddit client"}

        try:
            submission = reddit.submission(id=post_id)
            comment = submission.reply(text)
            return {
                "success": True,
                "id": comment.id,
                "url": f"https://reddit.com{comment.permalink}",
            }
        except Exception as e:
            return {"success": False, "error": str(e)}



#!/usr/bin/env python3
"""
Socialia MCP Server.

Provides MCP tools that delegate to CLI commands for reproducibility.
All MCP tool calls can be reproduced via CLI commands.

Environment Variables:
    SOCIALIA_ENV_FILE: Path to .env file to load (optional)
    Or set individual SOCIALIA_* env vars (see socialia setup)
"""

import os
import subprocess
import sys
import json
from pathlib import Path
from typing import Any


def load_env_file(env_file: str) -> None:
    """Load environment variables from a file."""
    path = Path(os.path.expandvars(env_file)).expanduser()
    if not path.exists():
        print(f"Warning: SOCIALIA_ENV_FILE not found: {path}", file=sys.stderr)
        return

    with open(path) as f:
        for line in f:
            line = line.strip()
            # Skip comments and empty lines
            if not line or line.startswith("#"):
                continue
            # Handle 'export VAR=value' or 'VAR=value'
            if line.startswith("export "):
                line = line[7:]
            if "=" in line:
                key, _, value = line.partition("=")
                key = key.strip()
                value = value.strip().strip('"').strip("'")
                # Expand $HOME and other vars
                value = os.path.expandvars(value)
                os.environ[key] = value


# Load env file if specified
if env_file := os.environ.get("SOCIALIA_ENV_FILE"):
    load_env_file(env_file)

try:
    from mcp.server import Server
    from mcp.server.stdio import stdio_server
    from mcp.types import Tool, TextContent

    HAS_MCP = True
except ImportError:
    HAS_MCP = False


def run_cli(*args: str) -> dict[str, Any]:
    """
    Run socialia CLI command and return result.

    This ensures all MCP operations are reproducible via CLI.
    """
    cmd = [sys.executable, "-m", "socialia.cli", "--json", *args]
    result = subprocess.run(cmd, capture_output=True, text=True)

    cli_command = f"socialia {' '.join(args)}"

    if result.returncode == 0:
        try:
            data = json.loads(result.stdout)
            data["cli_command"] = cli_command
            return data
        except json.JSONDecodeError:
            return {
                "success": True,
                "output": result.stdout,
                "cli_command": cli_command,
            }
    else:
        return {
            "success": False,
            "error": result.stderr or result.stdout,
            "cli_command": cli_command,
        }


def create_server() -> "Server":
    """Create and configure the MCP server."""
    if not HAS_MCP:
        raise ImportError("MCP package not installed. Run: pip install mcp")

    server = Server("socialia")

    @server.list_tools()
    async def list_tools() -> list[Tool]:
        """List available tools."""
        return [
            Tool(
                name="social_post",
                description="Post content to social media. CLI: socialia post <platform> <text>",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "platform": {
                            "type": "string",
                            "enum": ["twitter", "linkedin", "reddit", "youtube"],
                            "description": "Target platform",
                        },
                        "text": {
                            "type": "string",
                            "description": "Content to post",
                        },
                        "reply_to": {
                            "type": "string",
                            "description": "Post ID to reply to (optional)",
                        },
                        "dry_run": {
                            "type": "boolean",
                            "description": "Preview without posting",
                            "default": False,
                        },
                    },
                    "required": ["platform", "text"],
                },
            ),
            Tool(
                name="social_delete",
                description="Delete a social media post. CLI: socialia delete <platform> <post_id>",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "platform": {
                            "type": "string",
                            "enum": ["twitter", "linkedin", "reddit", "youtube"],
                            "description": "Target platform",
                        },
                        "post_id": {
                            "type": "string",
                            "description": "Post ID to delete",
                        },
                    },
                    "required": ["platform", "post_id"],
                },
            ),
            Tool(
                name="analytics_track",
                description="Track custom event in Google Analytics. CLI: socialia analytics track <event_name>",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "event_name": {
                            "type": "string",
                            "description": "Event name to track",
                        },
                        "params": {
                            "type": "object",
                            "description": "Event parameters (key-value pairs)",
                        },
                    },
                    "required": ["event_name"],
                },
            ),
            Tool(
                name="analytics_pageviews",
                description="Get page view metrics from Google Analytics. CLI: socialia analytics pageviews",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "start_date": {
                            "type": "string",
                            "description": "Start date (YYYY-MM-DD or '7daysAgo')",
                            "default": "7daysAgo",
                        },
                        "end_date": {
                            "type": "string",
                            "description": "End date (YYYY-MM-DD or 'today')",
                            "default": "today",
                        },
                        "path": {
                            "type": "string",
                            "description": "Optional page path filter",
                        },
                    },
                },
            ),
            Tool(
                name="analytics_sources",
                description="Get traffic sources from Google Analytics. CLI: socialia analytics sources",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "start_date": {
                            "type": "string",
                            "description": "Start date (YYYY-MM-DD or '7daysAgo')",
                            "default": "7daysAgo",
                        },
                        "end_date": {
                            "type": "string",
                            "description": "End date (YYYY-MM-DD or 'today')",
                            "default": "today",
                        },
                    },
                },
            ),
            Tool(
                name="analytics_realtime",
                description="Get realtime active users from Google Analytics. CLI: socialia analytics realtime",
                inputSchema={
                    "type": "object",
                    "properties": {},
                },
            ),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: dict) -> list[TextContent]:
        """Handle tool calls by delegating to CLI."""
        if name == "social_post":
            args = ["post", arguments["platform"], arguments["text"]]
            if arguments.get("reply_to"):
                args.extend(["--reply-to", arguments["reply_to"]])
            if arguments.get("dry_run"):
                args.append("--dry-run")
            result = run_cli(*args)

        elif name == "social_delete":
            result = run_cli("delete", arguments["platform"], arguments["post_id"])

        elif name == "analytics_track":
            args = ["analytics", "track", arguments["event_name"]]
            params = arguments.get("params", {})
            for key, value in params.items():
                args.extend(["--param", key, str(value)])
            result = run_cli(*args)

        elif name == "analytics_pageviews":
            args = ["analytics", "pageviews"]
            if arguments.get("start_date"):
                args.extend(["--start", arguments["start_date"]])
            if arguments.get("end_date"):
                args.extend(["--end", arguments["end_date"]])
            if arguments.get("path"):
                args.extend(["--path", arguments["path"]])
            result = run_cli(*args)

        elif name == "analytics_sources":
            args = ["analytics", "sources"]
            if arguments.get("start_date"):
                args.extend(["--start", arguments["start_date"]])
            if arguments.get("end_date"):
                args.extend(["--end", arguments["end_date"]])
            result = run_cli(*args)

        elif name == "analytics_realtime":
            result = run_cli("analytics", "realtime")

        else:
            result = {"success": False, "error": f"Unknown tool: {name}"}

        return [TextContent(type="text", text=json.dumps(result, indent=2))]

    return server


async def main():
    """Run the MCP server."""
    if not HAS_MCP:
        print("Error: MCP package not installed. Run: pip install mcp", file=sys.stderr)
        sys.exit(1)

    server = create_server()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream, write_stream, server.create_initialization_options()
        )


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



"""Twitter/X API v2 poster."""

from typing import Optional
from requests_oauthlib import OAuth1Session

from .base import BasePoster
from ._branding import get_env


class Twitter(BasePoster):
    """Twitter/X API v2 client using OAuth 1.0a."""

    POST_ENDPOINT = "https://api.x.com/2/tweets"
    DELETE_ENDPOINT = "https://api.x.com/2/tweets/{tweet_id}"

    def __init__(
        self,
        consumer_key: Optional[str] = None,
        consumer_secret: Optional[str] = None,
        access_token: Optional[str] = None,
        access_token_secret: Optional[str] = None,
    ):
        self.consumer_key = consumer_key or get_env("X_CONSUMER_KEY")
        self.consumer_secret = consumer_secret or get_env("X_CONSUMER_KEY_SECRET")
        self.access_token = access_token or get_env("X_ACCESSTOKEN")
        self.access_token_secret = access_token_secret or get_env(
            "X_ACCESSTOKEN_SECRET"
        )

    def _get_session(self) -> OAuth1Session:
        """Create OAuth1 session."""
        return OAuth1Session(
            self.consumer_key,
            client_secret=self.consumer_secret,
            resource_owner_key=self.access_token,
            resource_owner_secret=self.access_token_secret,
        )

    def validate_credentials(self) -> bool:
        """Check if all credentials are set."""
        return all(
            [
                self.consumer_key,
                self.consumer_secret,
                self.access_token,
                self.access_token_secret,
            ]
        )

    def post(
        self,
        text: str,
        reply_to: Optional[str] = None,
        quote_tweet_id: Optional[str] = None,
    ) -> dict:
        """
        Post a tweet.

        Args:
            text: Tweet content (max 280 chars standard, 25000 premium)
            reply_to: Tweet ID to reply to
            quote_tweet_id: Tweet ID to quote

        Returns:
            dict with 'success', 'id', 'url' or 'error'
        """
        if not self.validate_credentials():
            return {"success": False, "error": "Missing credentials"}

        oauth = self._get_session()
        payload = {"text": text}

        if reply_to:
            payload["reply"] = {"in_reply_to_tweet_id": reply_to}
        if quote_tweet_id:
            payload["quote_tweet_id"] = quote_tweet_id

        response = oauth.post(self.POST_ENDPOINT, json=payload)

        if response.status_code == 201:
            data = response.json()
            tweet_id = data["data"]["id"]
            return {
                "success": True,
                "id": tweet_id,
                "url": f"https://x.com/i/web/status/{tweet_id}",
            }
        else:
            return {
                "success": False,
                "error": f"{response.status_code}: {response.text}",
            }

    def delete(self, post_id: str) -> dict:
        """
        Delete a tweet.

        Args:
            post_id: Tweet ID to delete

        Returns:
            dict with 'success' and 'deleted' or 'error'
        """
        if not self.validate_credentials():
            return {"success": False, "error": "Missing credentials"}

        oauth = self._get_session()
        url = self.DELETE_ENDPOINT.format(tweet_id=post_id)
        response = oauth.delete(url)

        if response.status_code == 200:
            return {"success": True, "deleted": True}
        else:
            return {
                "success": False,
                "error": f"{response.status_code}: {response.text}",
            }

    def post_thread(self, tweets: list[str]) -> dict:
        """
        Post a thread of tweets.

        Args:
            tweets: List of tweet texts

        Returns:
            dict with 'success', 'ids', 'urls' or 'error'
        """
        ids = []
        urls = []
        reply_to = None

        for i, text in enumerate(tweets):
            result = self.post(text, reply_to=reply_to)
            if result["success"]:
                ids.append(result["id"])
                urls.append(result["url"])
                reply_to = result["id"]
            else:
                return {
                    "success": False,
                    "error": f"Failed at tweet {i + 1}: {result['error']}",
                    "partial_ids": ids,
                }

        return {"success": True, "ids": ids, "urls": urls}



"""LinkedIn API client."""

from typing import Optional
import requests

from .base import BasePoster
from ._branding import get_env


class LinkedIn(BasePoster):
    """LinkedIn API client using OAuth 2.0."""

    BASE_URL = "https://api.linkedin.com/v2"
    ME_ENDPOINT = f"{BASE_URL}/me"
    USERINFO_ENDPOINT = f"{BASE_URL}/userinfo"  # OpenID Connect endpoint
    UGC_POSTS_ENDPOINT = f"{BASE_URL}/ugcPosts"
    POSTS_ENDPOINT = f"{BASE_URL}/posts"

    def __init__(
        self,
        access_token: Optional[str] = None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
    ):
        self.access_token = access_token or get_env("LINKEDIN_ACCESS_TOKEN")
        self.client_id = client_id or get_env("LINKEDIN_CLIENT_ID")
        self.client_secret = client_secret or get_env("LINKEDIN_CLIENT_SECRET")
        self._user_urn: Optional[str] = None

    def _get_headers(self) -> dict:
        """Get headers for LinkedIn API requests."""
        return {
            "Authorization": f"Bearer {self.access_token}",
            "X-Restli-Protocol-Version": "2.0.0",
            "LinkedIn-Version": "202501",
            "Content-Type": "application/json",
        }

    def validate_credentials(self) -> bool:
        """Check if access token is set."""
        return bool(self.access_token)

    def _get_user_urn(self) -> Optional[str]:
        """Get the authenticated user's URN."""
        if self._user_urn:
            return self._user_urn

        if not self.validate_credentials():
            return None

        # Try OpenID Connect userinfo endpoint first (requires openid scope)
        response = requests.get(
            self.USERINFO_ENDPOINT,
            headers={"Authorization": f"Bearer {self.access_token}"},
        )
        if response.status_code == 200:
            user_data = response.json()
            # OpenID returns 'sub' as the user identifier
            if "sub" in user_data:
                self._user_urn = f"urn:li:person:{user_data['sub']}"
                return self._user_urn

        # Fallback to /me endpoint (requires r_liteprofile scope)
        response = requests.get(self.ME_ENDPOINT, headers=self._get_headers())
        if response.status_code == 200:
            user_data = response.json()
            self._user_urn = f"urn:li:person:{user_data['id']}"
            return self._user_urn

        return None

    def post(
        self,
        text: str,
        visibility: str = "PUBLIC",
    ) -> dict:
        """
        Post to LinkedIn feed.

        Args:
            text: Post content
            visibility: PUBLIC, CONNECTIONS, or LOGGED_IN

        Returns:
            dict with 'success', 'id', 'url' or 'error'
        """
        if not self.validate_credentials():
            return {"success": False, "error": "Missing access token"}

        author_urn = self._get_user_urn()
        if not author_urn:
            return {"success": False, "error": "Could not get user URN"}

        # Using UGC Posts API (more widely available)
        post_data = {
            "author": author_urn,
            "lifecycleState": "PUBLISHED",
            "specificContent": {
                "com.linkedin.ugc.ShareContent": {
                    "shareCommentary": {"text": text},
                    "shareMediaCategory": "NONE",
                }
            },
            "visibility": {"com.linkedin.ugc.MemberNetworkVisibility": visibility},
        }

        response = requests.post(
            self.UGC_POSTS_ENDPOINT, headers=self._get_headers(), json=post_data
        )

        if response.status_code == 201:
            post_id = response.headers.get("X-RestLi-Id", "")
            return {
                "success": True,
                "id": post_id,
                "url": f"https://www.linkedin.com/feed/update/{post_id}/",
            }
        else:
            return {
                "success": False,
                "error": f"{response.status_code}: {response.text}",
            }

    def delete(self, post_id: str) -> dict:
        """
        Delete a LinkedIn post.

        Note: LinkedIn API has limited delete support.

        Args:
            post_id: Post URN/ID to delete

        Returns:
            dict with 'success' or 'error'
        """
        if not self.validate_credentials():
            return {"success": False, "error": "Missing access token"}

        # LinkedIn delete endpoint
        url = f"{self.UGC_POSTS_ENDPOINT}/{post_id}"
        response = requests.delete(url, headers=self._get_headers())

        if response.status_code in (200, 204):
            return {"success": True, "deleted": True}
        else:
            return {
                "success": False,
                "error": f"{response.status_code}: {response.text}",
            }

    def get_token_info(self) -> dict:
        """Get information about the current access token."""
        if not self.validate_credentials():
            return {"valid": False, "error": "No access token"}

        response = requests.get(self.ME_ENDPOINT, headers=self._get_headers())
        if response.status_code == 200:
            return {"valid": True, "user": response.json()}
        else:
            return {"valid": False, "error": response.text}



"""Tests for Twitter client."""

from unittest.mock import MagicMock

from socialia.twitter import Twitter


class TestTwitter:
    """Test Twitter class."""

    def test_init_with_credentials(self, twitter_credentials):
        """Test initialization with explicit credentials."""
        client = Twitter(**twitter_credentials)
        assert client.consumer_key == "test_consumer_key"
        assert client.consumer_secret == "test_consumer_secret"
        assert client.access_token == "test_access_token"
        assert client.access_token_secret == "test_access_token_secret"

    def test_init_from_environment(self, monkeypatch):
        """Test initialization from environment variables."""
        # Clear any existing env vars first
        for prefix in ["SOCIALIA_", "SCITEX_"]:
            monkeypatch.delenv(f"{prefix}X_CONSUMER_KEY", raising=False)
            monkeypatch.delenv(f"{prefix}X_CONSUMER_KEY_SECRET", raising=False)
            monkeypatch.delenv(f"{prefix}X_ACCESSTOKEN", raising=False)
            monkeypatch.delenv(f"{prefix}X_ACCESSTOKEN_SECRET", raising=False)

        monkeypatch.setenv("SOCIALIA_X_CONSUMER_KEY", "env_consumer_key")
        monkeypatch.setenv("SOCIALIA_X_CONSUMER_KEY_SECRET", "env_consumer_secret")
        monkeypatch.setenv("SOCIALIA_X_ACCESSTOKEN", "env_access_token")
        monkeypatch.setenv("SOCIALIA_X_ACCESSTOKEN_SECRET", "env_access_secret")

        client = Twitter()
        assert client.consumer_key == "env_consumer_key"
        assert client.consumer_secret == "env_consumer_secret"

    def test_validate_credentials_valid(self, twitter_credentials):
        """Test credential validation with valid credentials."""
        client = Twitter(**twitter_credentials)
        assert client.validate_credentials() is True

    def test_validate_credentials_missing(self, monkeypatch):
        """Test credential validation with missing credentials."""
        # Clear environment variables for both prefixes
        for prefix in ["SOCIALIA_", "SCITEX_"]:
            monkeypatch.delenv(f"{prefix}X_CONSUMER_KEY", raising=False)
            monkeypatch.delenv(f"{prefix}X_CONSUMER_KEY_SECRET", raising=False)
            monkeypatch.delenv(f"{prefix}X_ACCESSTOKEN", raising=False)
            monkeypatch.delenv(f"{prefix}X_ACCESSTOKEN_SECRET", raising=False)
        client = Twitter(consumer_key="only_one")
        assert client.validate_credentials() is False

    def test_post_missing_credentials(self, monkeypatch):
        """Test post fails with missing credentials."""
        # Clear environment variables for both prefixes
        for prefix in ["SOCIALIA_", "SCITEX_"]:
            monkeypatch.delenv(f"{prefix}X_CONSUMER_KEY", raising=False)
            monkeypatch.delenv(f"{prefix}X_CONSUMER_KEY_SECRET", raising=False)
            monkeypatch.delenv(f"{prefix}X_ACCESSTOKEN", raising=False)
            monkeypatch.delenv(f"{prefix}X_ACCESSTOKEN_SECRET", raising=False)
        client = Twitter()
        result = client.post("Test")
        assert result["success"] is False
        assert "credentials" in result["error"].lower()

    def test_post_success(self, twitter_credentials, mock_oauth_session):
        """Test successful post."""
        mock_oauth_session.post.return_value = MagicMock(
            status_code=201,
            json=lambda: {"data": {"id": "12345"}},
        )

        client = Twitter(**twitter_credentials)
        result = client.post("Hello World!")

        assert result["success"] is True
        assert result["id"] == "12345"
        assert "x.com" in result["url"]

    def test_post_failure(self, twitter_credentials, mock_oauth_session):
        """Test failed post."""
        mock_oauth_session.post.return_value = MagicMock(
            status_code=403,
            text="Forbidden",
        )

        client = Twitter(**twitter_credentials)
        result = client.post("Hello World!")

        assert result["success"] is False
        assert "403" in result["error"]

    def test_post_with_reply(self, twitter_credentials, mock_oauth_session):
        """Test post with reply_to."""
        mock_oauth_session.post.return_value = MagicMock(
            status_code=201,
            json=lambda: {"data": {"id": "67890"}},
        )

        client = Twitter(**twitter_credentials)
        result = client.post("Reply text", reply_to="12345")

        assert result["success"] is True
        call_args = mock_oauth_session.post.call_args
        assert "reply" in call_args.kwargs["json"]

    def test_delete_success(self, twitter_credentials, mock_oauth_session):
        """Test successful delete."""
        mock_oauth_session.delete.return_value = MagicMock(status_code=200)

        client = Twitter(**twitter_credentials)
        result = client.delete("12345")

        assert result["success"] is True
        assert result["deleted"] is True

    def test_delete_failure(self, twitter_credentials, mock_oauth_session):
        """Test failed delete."""
        mock_oauth_session.delete.return_value = MagicMock(
            status_code=404,
            text="Not Found",
        )

        client = Twitter(**twitter_credentials)
        result = client.delete("invalid_id")

        assert result["success"] is False
        assert "404" in result["error"]

    def test_post_thread_success(self, twitter_credentials, mock_oauth_session):
        """Test successful thread posting."""
        mock_oauth_session.post.side_effect = [
            MagicMock(status_code=201, json=lambda: {"data": {"id": "1"}}),
            MagicMock(status_code=201, json=lambda: {"data": {"id": "2"}}),
            MagicMock(status_code=201, json=lambda: {"data": {"id": "3"}}),
        ]

        client = Twitter(**twitter_credentials)
        result = client.post_thread(["First", "Second", "Third"])

        assert result["success"] is True
        assert len(result["ids"]) == 3
        assert len(result["urls"]) == 3

    def test_post_thread_partial_failure(self, twitter_credentials, mock_oauth_session):
        """Test thread posting with partial failure."""
        mock_oauth_session.post.side_effect = [
            MagicMock(status_code=201, json=lambda: {"data": {"id": "1"}}),
            MagicMock(status_code=403, text="Rate limited"),
        ]

        client = Twitter(**twitter_credentials)
        result = client.post_thread(["First", "Second", "Third"])

        assert result["success"] is False
        assert "partial_ids" in result
        assert len(result["partial_ids"]) == 1



"""Base class for social media posters."""

from abc import ABC, abstractmethod


class BasePoster(ABC):
    """Abstract base class for social media posters."""

    @abstractmethod
    def post(self, text: str, **kwargs) -> dict:
        """Post content to the platform."""
        pass

    @abstractmethod
    def delete(self, post_id: str) -> dict:
        """Delete a post by ID."""
        pass

    def validate_credentials(self) -> bool:
        """Check if credentials are configured."""
        return True
